@@ -9,7 +9,7 @@
 
 ---
 
-## ⚡ Quick Start
+## Quick Start
 
 Install from PyPI:
 
@@ -33,13 +33,13 @@ class Person(CStruct):
     name: Annotated[str, CStr(6)]
 
 
-person = Person.c_build(bytes([18, 170]) + b"Pippo\x00")
+person = Person.c_decode(bytes([18, 170]) + b"Pippo\x00")
 print(person)  # Person(info=Info(age=18, height=170), name='Pippo')
 ```
 
 ---
 
-## 🚀 Introduction
+## Introduction
 
 `cstructimpl` makes working with binary data in Python simple and intuitive.
 By subclassing `CStruct`, you can define Python classes that map directly to C-style `struct`s and parse raw bytes into fully typed objects.
@@ -48,7 +48,7 @@ No manual parsing, no boilerplate — just define your struct and let the librar
 
 ---
 
-## 🔧 Type System
+## Type System
 
 At the core of the library is the `BaseType` protocol, which defines how types behave in the C world:
 
@@ -59,13 +59,21 @@ class BaseType(Protocol[T]):
     def c_align(self) -> int: ...
     def c_signed(self) -> bool: ...
 
-    def c_build(
+    def c_decode(
         self,
         raw: bytes,
         *,
-        byteorder: Literal["little", "big"] = "little",
+        is_little_endian: bool = True,
         signed: bool | None = None,
     ) -> T | None: ...
+
+    def c_encode(
+        self,
+        data: T,
+        *,
+        is_little_endian: bool = True,
+        signed: bool | None = None,
+    ) -> bytes: ...
 ```
 
 Any class that follows this protocol can act as a `BaseType`, controlling its own parsing, size, and alignment.
@@ -74,16 +82,17 @@ When parsing a struct:
 
 - If a field type is itself a `BaseType`, parsing happens automatically.
 - Otherwise, annotate the field with `Annotated[..., BaseType]` to tell the parser how to interpret it.
+- Types such as `int` have a default converter for a `BaseType` if no annotation is provided. If you want to change this behavior you need to ovveride them in the following dictionary `cstructimpl.c_lib.DEFAULT_TYPE_TO_BASETYPE`.
 
 The library comes with a set of ready-to-use type definitions that cover the majority of C primitive types.
 
 ---
 
-## 📌 Examples
+## Examples
 
 Here are a few practical examples showing how `cstructimpl` works in real-world scenarios.
 
-### Basic Struct
+### Basic Deserialization
 
 Define a simple struct with two fields:
 
@@ -95,7 +104,22 @@ class Point(CStruct):
 
 assert Point.c_size() == 2
 assert Point.c_align() == 1
-assert Point.c_build(bytes([1, 2])) == Point(1, 2)
+assert Point.c_decode(bytes([1, 2])) == Point(1, 2)
+```
+
+---
+
+### Serializing a Class
+
+Create a class instance and serlialize it to raw bytes
+
+```python
+class Rect(CStruct):
+    width: Annotated[int, CType.U8]
+    height: Annotated[int, CType.U8] = 10
+
+rect = Rect(2)
+assert rect.c_encode() == bytes([2, 10])
 ```
 
 ---
@@ -117,7 +141,7 @@ class Rectangle(CStruct):
 
 assert Rectangle.c_size() == 4
 assert Rectangle.c_align() == 2
-assert Rectangle.c_build(bytes([1, 0, 2, 3])) == Rectangle(1, Dimensions(2, 3))
+assert Rectangle.c_decode(bytes([1, 0, 2, 3])) == Rectangle(1, Dimensions(2, 3))
 ```
 
 ---
@@ -133,7 +157,7 @@ class Message(CStruct):
 
 
 raw = bytes([5, 0]) + b"Helo\x00"
-assert Message.c_build(raw) == Message(5, "Helo")
+assert Message.c_decode(raw) == Message(5, "Helo")
 ```
 
 ---
@@ -154,7 +178,7 @@ class Person(CStruct):
 
 
 raw = bytes([18, 0, 1, 0])
-assert Person.c_build(raw) == Person(18, Mood.SAD)
+assert Person.c_decode(raw) == Person(18, Mood.SAD)
 ```
 
 ---
@@ -175,7 +199,7 @@ class ItemList(CStruct):
 
 
 data = bytes(range(1, 13))  # 3 items × 4 bytes each
-parsed = ItemList.c_build(data)
+parsed = ItemList.c_decode(data)
 
 assert parsed == ItemList([
     Item(1, 2, 3),
@@ -184,10 +208,47 @@ assert parsed == ItemList([
 ])
 ```
 
+### Custom BaseType
+
+> > Hey! Is there a type that serializes an hash-map of list of structs of ...?
+>
+> > Yeah, sure there is! You can do it yourself!
+
+`cstructimpl` lets you define your own `BaseType` implementations to handle any kind of data that is not present among the built-in primitives.
+
+For example, here's a custom type that interprets a raw integer as a **Unix timestamp**, returning a Python `datetime` object:
+
+```python
+class UnixTimestamp(BaseType[datetime]):
+    def c_size(self) -> int:
+        return 4
+
+    def c_align(self) -> int:
+        return 4
+
+    def c_signed(self) -> bool:
+        return False
+
+    def c_decode(self, raw: bytes, *, byteorder="little", signed=False) -> datetime:
+        ts = int.from_bytes(raw, byteorder=byteorder, signed=signed)
+        return datetime.utcfromtimestamp(ts)
+
+    def c_encode(self): pass
+
+
+    @dataclass
+    class LogEntry(CStruct):
+        timestamp: Annotated[datetime, UnixTimestamp()]
+        level: Annotated[int, CType.U8]
+
+
+    parsed = LogEntry.c_decode(bytes([255, 0, 0, 0, 3, 0, 0, 0]))
+    assert parsed == LogEntry(datetime.fromtimestamp(255), 3)
+```
+
 ---
 
-
-## 🎭 Autocast
+## Autocast
 
 Sometimes raw numeric values carry semantic meaning. In C, this is usually handled with `enum`s.
 With `cstructimpl`, you can automatically reinterpret values into enums (or other types) using `Autocast`.
@@ -218,7 +279,7 @@ class ResultType(Enum):
 
 
 class Person(CStruct):
-    kind: Annotated[ResultType, CBuilder(CType.U8, lambda u8: ResultType(u8))]
+    kind: Annotated[ResultType, CMapper(CType.U8, lambda u8: ResultType(u8))]
     error_code: Annotated[int, CType.I32]
 ```
 
@@ -226,17 +287,18 @@ But much simpler and less error-prone.
 
 ---
 
-## ✨ Features
+## Features
 
 - Define Python classes that map directly to C `struct`s
 - Parse raw bytes into typed objects with a single method call
+- Serialize a class to raw bytes using built-in type system
 - Built-in type system for common C primitives
 - Support for nested structs
 - Flexible extension via the `BaseType` protocol
 
 ---
 
-## 📖 Use Cases
+## Use Cases
 
 - Parsing binary network protocols
 - Working with binary file formats
@@ -245,13 +307,13 @@ But much simpler and less error-prone.
 
 ---
 
-## 📚 Documentation
+## Documentation
 
 More detailed usage examples and advanced topics are available in the [documentation](https://github.com/Brendon-Mendicino/cstructimpl/wiki).
 
 ---
 
-## 🤝 Contributing
+## Contributing
 
 Contributions are welcome!
 
@@ -259,7 +321,7 @@ If you'd like to improve `cstructimpl`, please open an issue or submit a pull re
 
 ---
 
-## 📜 License
+## License
 
 This project is licensed under the terms of the [Apache-2.0 License](https://github.com/Brendon-Mendicino/cstructimpl/blob/main/LICENSE).
 
@@ -267,8 +329,19 @@ This project is licensed under the terms of the [Apache-2.0 License](https://git
 
 from . import c_lib
 from . import c_types
+from . import c_annotations
 
-from .c_types import BaseType, HasBaseType, CType, CArray, CPadding, CStr, CBuilder
+from .c_types import (
+    BaseType,
+    HasBaseType,
+    CType,
+    CArray,
+    CPadding,
+    CStr,
+    CMapper,
+    CBool,
+    CFloat,
+)
 from .c_lib import c_struct, CStruct
 from .c_annotations import Autocast
 
@@ -276,13 +349,16 @@ from .c_annotations import Autocast
 __all__ = [
     "c_lib",
     "c_types",
+    "c_annotations",
     "BaseType",
     "HasBaseType",
     "CType",
     "CArray",
     "CPadding",
     "CStr",
-    "CBuilder",
+    "CMapper",
+    "CBool",
+    "CFloat",
     "c_struct",
     "CStruct",
     "Autocast",
