@@ -1,13 +1,19 @@
 """Decorators and classes to extend types."""
 
+import inspect
 import sys
 
 from dataclasses import Field, dataclass, fields, is_dataclass
 from itertools import islice
-from typing import Callable, Generic, Literal, TypeVar, get_origin
+from typing import (
+    Callable,
+    TypeVar,
+    get_origin,
+)
 
+from .util import hybridmethod
 from .c_annotations import Autocast
-from .c_types import BaseType, CBuilder, HasBaseType, CPadding
+from .c_types import BaseType, CBool, CMapper, HasBaseType, CPadding, CType, CFloat
 
 if sys.version_info >= (3, 11):
     from typing import Self
@@ -17,6 +23,17 @@ else:
 T = TypeVar("T")
 
 
+DEFAULT_TYPE_TO_BASETYPE = {
+    bool: CBool(),
+    int: CType.I32,
+    float: CFloat.F32,
+}
+
+
+class _MarkerPadding(CPadding):
+    pass
+
+
 def _get_origin(t: type) -> type:
     origin = getattr(t, "__origin__", t)
     return get_origin(origin) or origin
@@ -34,13 +51,25 @@ def _get_field_metadata(f: Field) -> tuple | None:
     return _get_metadata(f.type)
 
 
-def _is_autocast(f: Field) -> bool:
+def _is_autocast(f: Field) -> Autocast | None:
     autocast = _get_field_metadata(f) or tuple()
-    autocast = map(lambda f: isinstance(f, Autocast), autocast)
-    return any(autocast)
+    autocast = filter(lambda f: isinstance(f, Autocast), autocast)
+    return next(autocast, None)
 
 
-def _get_ctype(t: Field) -> BaseType | None:
+def _get_ctype_decode_type(ctype: BaseType[T]) -> type[T]:
+    ret_type = inspect.signature(ctype.c_decode).return_annotation
+    ret_type = _get_origin(ret_type)
+    return ret_type
+
+
+def _get_default_basetype(t: type) -> BaseType | None:
+    return DEFAULT_TYPE_TO_BASETYPE.get(t, None)
+
+
+def _get_basetype(t: Field) -> BaseType | None:
+    """Get the `BaseType` of a `Field`."""
+
     origin = _get_field_origin(t)
     metadata = _get_field_metadata(t) or tuple()
 
@@ -59,7 +88,9 @@ def _get_ctype(t: Field) -> BaseType | None:
     if isinstance(origin, HasBaseType):
         return origin.c_get_type()
 
-    return None
+    # If no metadata is provided try and check if the current
+    # type has a default `BaseType`
+    return _get_default_basetype(origin)
 
 
 def _types_from_dataclass(cls: type) -> list[BaseType]:
@@ -67,7 +98,7 @@ def _types_from_dataclass(cls: type) -> list[BaseType]:
 
     for field in fields(cls):
         autocast = _is_autocast(field)
-        ctype = _get_ctype(field)
+        ctype = _get_basetype(field)
 
         if ctype is None:
             raise ValueError(
@@ -75,14 +106,15 @@ def _types_from_dataclass(cls: type) -> list[BaseType]:
             )
 
         if autocast:
-            field_type = _get_field_origin(field)
+            cls_field_type = _get_field_origin(field)
+            ret_type = _get_ctype_decode_type(ctype)
 
-            if not isinstance(field_type, type):
+            if not isinstance(cls_field_type, type):
                 raise ValueError(
-                    f"Autocast is set to True, but the dataclass field is not an instance of `type`! Cannot cast serialized object to the field type. Either disable autocast or change type signature. {field_type=}"
+                    f"Autocast is set to True, but the dataclass field is not an instance of `type`! Cannot cast serialized object to the field type. Either disable autocast or change type signature. {cls_field_type=}"
                 )
 
-            ctype = CBuilder(ctype, field_type)
+            ctype = CMapper(ctype, cls_field_type, ret_type)
 
         ctypes.append(ctype)
 
@@ -107,7 +139,7 @@ class _Pipeline:
 
 
 @dataclass
-class _StructTypeHandler(Generic[T]):
+class _StructTypeHandler(BaseType[T]):
     """StructTypeHandler"""
 
     pipeline: _Pipeline
@@ -123,11 +155,11 @@ class _StructTypeHandler(Generic[T]):
     def c_signed(self) -> bool:
         raise NotImplementedError()
 
-    def c_build(
+    def c_decode(
         self,
         raw: bytes,
         *,
-        byteorder: Literal["little", "big"] = "little",
+        is_little_endian: bool = True,
         signed: bool | None = None,
     ) -> T:
         # TODO: handle byteorder, signed, size, align
@@ -136,25 +168,46 @@ class _StructTypeHandler(Generic[T]):
         cls_items = []
 
         for pipe_item in self.pipeline.pipeline:
-            raw_bytes = islice(raw_slice, pipe_item.c_size())
+            raw_bytes = bytes(islice(raw_slice, pipe_item.c_size()))
 
-            cls_item = pipe_item.c_build(
-                bytes(raw_bytes),
-                byteorder=byteorder,
+            if isinstance(pipe_item, _MarkerPadding):
+                continue
+
+            cls_item = pipe_item.c_decode(
+                raw_bytes,
+                is_little_endian=is_little_endian,
                 signed=signed,
             )
 
-            if cls_item is not None:
-                cls_items.append(cls_item)
+            cls_items.append(cls_item)
 
         if self.strict:
             _strict_dataclass_fields_check(self.cls, cls_items)
 
         return self.cls(*cls_items)
 
+    def c_encode(
+        self, data: T, *, is_little_endian: bool = True, signed: bool | None = None
+    ) -> bytes:
+        raw_data = bytes()
+        data_fields = (getattr(data, f.name) for f in fields(data))
+
+        for pipe_item in self.pipeline.pipeline:
+            if isinstance(pipe_item, _MarkerPadding):
+                raw_data += pipe_item.c_encode(
+                    None, is_little_endian=is_little_endian, signed=signed
+                )
+                continue
+
+            raw_data += pipe_item.c_encode(
+                next(data_fields), is_little_endian=is_little_endian, signed=signed
+            )
+
+        return raw_data
+
 
 @dataclass
-class _UnionTypeHandler(Generic[T]):
+class _UnionTypeHandler(BaseType[T]):
     """StructTypeHandler"""
 
     pipeline: _Pipeline
@@ -170,11 +223,11 @@ class _UnionTypeHandler(Generic[T]):
     def c_signed(self) -> bool:
         raise NotImplementedError()
 
-    def c_build(
+    def c_decode(
         self,
         raw: bytes,
         *,
-        byteorder: Literal["little", "big"] = "little",
+        is_little_endian: bool = True,
         signed: bool | None = None,
     ) -> T:
         # TODO: handle byteorder, signed, size, align
@@ -184,9 +237,9 @@ class _UnionTypeHandler(Generic[T]):
         for pipe_item in self.pipeline.pipeline:
             raw_bytes = islice(raw, pipe_item.c_size())
 
-            cls_item = pipe_item.c_build(
+            cls_item = pipe_item.c_decode(
                 bytes(raw_bytes),
-                byteorder=byteorder,
+                is_little_endian=is_little_endian,
                 signed=signed,
             )
 
@@ -199,6 +252,11 @@ class _UnionTypeHandler(Generic[T]):
         # TODO: make union fields lazyly evalutated
         return self.cls(*cls_items)
 
+    def c_encode(
+        self, data: T, *, is_little_endian: bool = True, signed: bool | None = None
+    ) -> bytes:
+        pass
+
 
 def _build_struct_pipeline(
     ctypes: list[BaseType], *, override_align: int | None = None
@@ -211,15 +269,22 @@ def _build_struct_pipeline(
         padding = -current_size % ctype.c_align()
 
         if padding != 0:
-            pipeline.append(CPadding(padding))
+            pipeline.append(_MarkerPadding(padding))
+            current_size += padding
 
         current_align = max(current_align, ctype.c_align())
         current_size += ctype.c_size()
         pipeline.append(ctype)
 
+    # Override struct global padding
     if override_align is not None:
         current_align = override_align
 
+    # Add trailing padding if needed
+    padding = -current_size % current_align
+    if padding != 0:
+        pipeline.append(_MarkerPadding(padding))
+
     # A struct always needs to always have a size which is a mutiple of its
     # alignemt
     current_size += -current_size % current_align
@@ -286,6 +351,35 @@ def c_struct(
     union: bool = False,
     strict: bool = True,
 ) -> Callable[[type[T]], type[T]]:
+    """Decorator used to automatically implement the `BaseType`
+    methods in a class. Every class that is annotated with
+    this decorator, automatically becomes a `BaseType` itself.
+
+    This decorator converts the class in a `dataclass` it isn't already,
+    this is done in order to exploit the dataclasses utility
+    methods for the fields of the class.
+
+    Args:
+        align (int | None, optional): struct alignment. Defaults to None.
+        union (bool, optional): if true the fields of a struct are
+            interpreted as a C union. Defaults to False.
+        strict (bool, optional): check that the converted value in
+            a parameter matches the actual type defined in the class. Defaults to True.
+
+    Returns:
+        _ (Callable[[type[T]], type[T]]): returns the augmented class
+
+    Examples:
+
+    >>> @c_struct()
+    ... class Point:
+    ...     x: int
+    ...     y: int
+    >>> point_ctype = Point.c_get_type()
+    >>> p = point_ctype.c_decode(bytes([1, 0, 0, 0, 2, 0, 0, 0]))
+    >>> assert p == Point(1, 2)
+    """
+
     def c_struct_inner(cls: type[T]):
         if not union:
             return _c_struct(cls, align, strict)
@@ -297,9 +391,20 @@ def c_struct(
 
 class CStruct:
     """Overrides the attributes of a class, directly embedding the
-    method for a BaseType iside the type definition.
+    method for a `BaseType` iside the type definition.
 
-    This is a simple wrapper over the `c_struct` decorator."""
+    This is a simple wrapper over the `c_struct` decorator.
+
+    Args:
+
+    Examples:
+
+    >>> class Point(CStruct):
+    ...     x: int
+    ...     y: int
+    >>> p = Point.c_decode(bytes([1, 0, 0, 0, 2, 0, 0, 0]))
+    >>> assert p == Point(1, 2)
+    """
 
     def __init_subclass__(cls, **kwargs):
         new_cls = c_struct(**kwargs)(cls)
@@ -309,10 +414,13 @@ class CStruct:
         if sys.version_info >= (3, 12):
             attrs = BaseType.__protocol_attrs__
         else:
-            attrs = {"c_size", "c_align", "c_signed", "c_build"}
+            attrs = {"c_size", "c_align", "c_signed", "c_decode", "c_encode"}
 
         for attr in attrs:
-            setattr(cls, attr, getattr(base_type, attr))
+            if attr == "c_encode":
+                setattr(cls, "_c_encode", getattr(base_type, attr))
+            else:
+                setattr(cls, attr, getattr(base_type, attr))
 
     @classmethod
     def c_size(cls) -> int: ...
@@ -324,10 +432,30 @@ class CStruct:
     def c_signed(cls) -> bool: ...
 
     @classmethod
-    def c_build(
+    def c_decode(
         cls,
         raw: bytes,
         *,
-        byteorder: Literal["little", "big"] = "little",
+        is_little_endian: bool = True,
         signed: bool | None = None,
     ) -> Self: ...
+
+    # TODO: decide if this method should be split into two separate ones, one left as the signature of `BaseType` and another one that class this with data=self
+    @hybridmethod
+    def c_encode(
+        self,
+        data: Self | None = None,
+        *,
+        is_little_endian: bool = True,
+        signed: bool | None = None,
+    ) -> bytes:
+        if data is None and not isinstance(self, type):
+            data = self
+
+        if data is None:
+            raise ValueError("Data is None!")
+
+        return self._c_encode(data, is_little_endian=is_little_endian, signed=signed)
+
+    @classmethod
+    def c_get_type(cls) -> BaseType[Self]: ...
